/// # Box<T>
/// 可以理解就是void*，其指向存储在堆上的内存，因为RUST必须在编译器知道栈的分配，
/// 但是某些数据必须是运行时动态变化的，大小是不确定的，所以可以将其通过box（装箱），
/// 使用者在使用时将其拆箱进行使用，但是代码的组织上可以使用大小确定的Box<T>来占位。
pub fn learn() {
    let mut num = Box::new(5);

    *num = 7;
    println!("num = {}", num);

    /// 构造一个链表，链表的每个节点存储一个i32另外一个元素是另一个节点
    enum List {
        //Cons(i32, List), //直接使用List会导致无限递归，无法计算出List的大小
        Cons(i32, Box<List>), // 使用Box将装箱，使得可以实现该需求。因为Box的大小是确定，
        Nil,
    }
    let mut list = List::Cons(12, Box::new(List::Cons(13, Box::new(List::Nil))));

    loop {
        list = match list {
            List::Cons(num, next) => {
                println!("{},", num);
                *next
            }
            List::Nil => {
                break;
            }
        };
    }

    // Deref trait(Dereference operator) 解引用运算符，可以通过实现Deref trait来实现在使用上智能指针像是引用一样。
}
