use std::rc::Rc;

/// # RC<T>
/// RC<T>: Reference counting, 是一个带有引用计数的智能指针，其相较于Box来说就是可以同时被多个所有者拥有
/// 只有当所有的所有者的都生命周期结束其才会被释放。
/// RC提供的不可变引用。

pub fn learn() {
    enum List {
        Cons(i32, Rc<List>),
        Nil,
    }
    // 创建一个RC list。这里为了其能够被b和c
    let mut a = Rc::new(List::Cons(2, Rc::new(List::Cons(4, Rc::new(List::Nil)))));
    // 通过Rc::clone将数据的智能指针copy一份，内容还是一样的
    let b = List::Cons(3, Rc::clone(&a));
    println!(
        "strong ref of a is {}, weak is {}",
        Rc::strong_count(&a),
        Rc::weak_count(&a)
    );
    let c = List::Cons(3, Rc::clone(&a));
    println!(
        "strong ref of a is {}, weak is {}",
        Rc::strong_count(&a),
        Rc::weak_count(&a)
    );
}
