fn main() {
    println!("Hello, world!");
    smart_point_learn();
}

use std::cell::RefCell;
use std::rc::{Rc, Weak};

use boxt::SmartPoint::my_box;
use boxt::SmartPoint::rc_t;
use boxt::SmartPoint::BoxT;

fn smart_point_learn() {
    BoxT::learn();
    my_box::learn();
    rc_t::learn();

    #[derive(Debug)]
    struct StNode {
        data: String,
        //使用weak可以使得指向parent并非强引用不会影响parent的释放
        parent: RefCell<Weak<StNode>>,
        //使用rc可以使得强引用子节点，只有父节点不再持有时才可释放
        child: RefCell<Vec<Rc<StNode>>>,
    }
    let leaf = Rc::new(StNode{
        data: String::from("leaf"),
        parent: RefCell::new(Weak::new()),
        child: RefCell::new(vec![]),
    });
    println!("leaf parent = {:?}", leaf.parent.borrow().upgrade());
    let root = Rc::new(StNode{
        data: String::from("root"),
        parent: RefCell::new(Weak::new()),
        child:RefCell::new(vec![leaf.clone()]),
    });
    *(leaf.parent.borrow_mut()) = Rc::downgrade(&root);
    println!("leaf parent = {:?}", leaf.parent.borrow().upgrade());

    println!("root strong ref {}, weak ref {}", Rc::strong_count(&root), Rc::weak_count(&root));
    println!("leaf strong ref {}, weak ref {}", Rc::strong_count(&leaf), Rc::weak_count(&leaf));

    drop(root);
    println!("leaf strong ref {}, weak ref {}", Rc::strong_count(&leaf), Rc::weak_count(&leaf));
    println!("leaf parent = {:?}", leaf.parent.borrow().upgrade());
    // println!("tree is {}", root);
}
