//定义一个Draw trait，所有实现了Draw trait的类型都要具有draw方法。
pub trait Draw {
    fn draw(&self);
}

pub struct Screen {
    pub components: Vec<Box<dyn Draw>>,
}

impl Screen {
    pub fn new() -> Screen {
        Screen { components: vec![] }
    }
    pub fn add(&mut self, comp: Box<dyn Draw>) {
        self.components.push(comp)
    }
    pub fn run(&self) {
        if self.components.len() == 0 {
            println!("has no components!");
            return;
        }
        for obj in self.components.iter() {
            obj.draw();
        }
    }
}

pub struct Button {
    width: u32,
    height: u32,
    text: String,
}

impl Button {
    pub fn new(width: u32, height: u32, text: &str) -> Button {
        Button {
            width,
            height,
            text: String::from(text),
        }
    }
}

impl Draw for Button {
    fn draw(&self) {
        println!(
            "This is a Button, draw by {}x{} : {}",
            self.width, self.height, self.text
        );
    }
}
