fn main() {
    println!("Hello, world!");
    thread_learn();
    thread_learn1();
    thread_learn2();
}
use std::thread;
use std::time::Duration;

fn thread_learn() {
    let handle = thread::spawn(|| {
        for i in 1..5 {
            println!("thread {}", i);
            thread::sleep(Duration::from_millis(100));
        }
    });
    for i in 1..5 {
        println!("main {}", i);
        thread::sleep(Duration::from_millis(50));
    }
    handle.join().unwrap();
}

fn thread_learn1() {
    let v = vec![1, 2, 3, 4];
    //闭包是可以直接捕获上下文中的变量并使用的，但是在线程里不行。因为rust无法获知线程合适结束，也就
    //无法保证v的有效性。如果需要在threa里使用主线程中的变量。需要使用move将其移动到子线程，
    //这样主线程也就不在拥有v的所有权了

    /* 这种方式会编译不过。因为v不可以再子线程使用 */
    // let handle = thread::spawn(|| {
    //     println!("thread {:?}", v);
    // });
    // println!("main {:?}", v);

    /* 这种方式会编译不过。因为v被移动到子线程主线程已不可使用 */
    let handle = thread::spawn(move || {
        println!("thread {:?}", v);
    });
    //println!("main {:?}", v);
    handle.join().unwrap();
}

use std::sync::mpsc;
fn thread_learn2() {
    let (tx, rx) = mpsc::channel();
    let tx1 = mpsc::Sender::clone(&tx); //再产生一个生产者
    let handle = thread::spawn(move || {
        let val = String::from("hello");
        tx.send(val).unwrap();
        let msgs = vec!["aaaa", "qwer", "rty", "byebye", "zaas"];
        for msg in msgs {
            match tx.send(String::from(msg)) {
                Err(err) => {
                    println!("发送失败 {:?}", err);
                    break;
                }
                _ => {}
            }
            thread::sleep(Duration::from_micros(20));
        }
    });
    let thandle1 = thread::spawn(move || {
        let val = String::from("hello");
        tx1.send(val).unwrap();
        let msgs = vec!["aaaa1", "qwer1", "rty1", "byebye1", "zaas1"];
        for msg in msgs {
            match tx1.send(String::from(msg)) {
                Err(err) => {
                    println!("发送失败 {:?}", err);
                    break;
                }
                _ => {}
            }
            thread::sleep(Duration::from_micros(5));
        }
    });

    let handle1 = thread::spawn(move || loop {
        let val = match rx.try_recv() {
            Ok(msg) => msg,
            Err(_) => {
                thread::sleep(Duration::from_millis(10));
                continue;
            }
        };
        // let val = rx.recv().unwrap();
        if val == "byebye" {
            drop(rx);
            break;
        }
        println!("receive a msg: {}", val);
    });

    handle.join().unwrap();
    thandle1.join().unwrap();
    handle1.join().unwrap();
}
