use core::num;

use trait_learn::{self, NewsArticle, Summary, Tweet};
fn main() {
    println!("Hello, world!");
    trait_learn();
    largest_test();
    displaytest();
}


/* trait：RUST中实现一些接口或者特性或者共享功能的机制
   在C中我们一般对外提供功能都是使用的一个头文件去申明接口，然后在提供一个实现了这个接口的lib，
   这样外部的代码可根据接口进行调用实现功能，而内部的具体实现以及内部的数据结构就屏蔽在lib里，
   对于rust来说，对外提供的接口是通过pub关键字进行暴露的，也能够达到上述的目的，那么trait是做什么用呢？

   在我的理解trait是在接口之上的一个东西，也可以反过来理解，他定义了一个接口应当具备的功能。
   当我们定义某个trait后，所有实现了这个trait的数据结构他们在某种程度上就是有一种共同的功能，
   那么在实际使用时就可以利用这种共性去编写函数，减少对于不同类型相同功能的使用代码。

   比如我们都熟知的 print, 它就是可以将所有支持了Display这个trait的数据结构都进行打印到终端，
   我们只需在实现某个数据结构时将这个Display trait实现，其即可作为参数传递给print函数！
   就这个Display来说。其就类似于C++中某个类重载了<<操作符，或者python中重载了 _str_()一样。

   那么为什么要单独把trait这个概念领出来呢，而不是说像其他语言那样使用重载的概念，这是因为
   这个trait还可以作为参数的限制的一部分，也就是说，我们通过泛型可以接受各种数据类型，但是可能有些
   数据类型并不符合我们的要求，因为我们这个函数的具体实现完成的功能，必须要求类型具备某些属性才行，
   这时候在比如C++中可能会用模板的type list来限制，只有在list里面的类型才可以传入这个函数,
   类似这样
   template<T>
   typename enable_if<xxx>
   RUST是倒过来进行筛选，直接在泛型的标注上，标注只有具备这个trait的类型才可以被调用！
   类似这样
   T: impl xxx
   这样有什么好处？，我们可以不必为新增加了某个实现xxx的数据类型进行修改该函数的代码，
 */
fn trait_learn() {
    // 定义一个 新闻实例
    let news = NewsArticle {
        author:   String::from("ekko.bao"),
        content:  String::from("Th ShangHai Peenguins once again are the best hockey team in the NHL."),
        headline: String::from("Penguins win the stanley Cup Championship"),
        location: String::from("ShangHai, China"),
    };

    // 定义一个tweet实例
    let tweet = Tweet {
        username:   String::from("ekko.bao"),
        content:    String::from("of course, as you probably already know, people."),
        reply:      false,
        retweet:    false,
    };

    println!("1 new tweet: {}", tweet.summarize());
    println!("New article: {}", news.summarize());

    // 对于实现了 Display的类型可直接进行打印
    println!("tweet:{}", tweet);
    //NewsArticle未实现Diaply trait，无法进行打印。
    //println!("article:{}", news);

    //可以使用 notify ,因为 Tweet 和 NewsArticle 都实现了 Summary
    //trait_learn::notify(tweet);
    //trait_learn::notify(news);
    //无法使用notify：the trait bound `&str: Summary` is not satisfied
    //trait_learn::notify("new notify");

    // news和tweet 并不是同一种类型.是不可以调用的 notify2的
    //trait_learn::notify2(news, tweet);

    // news和tweet 都实现了Summary,可以调用notify1
    trait_learn::notify1(news, tweet);
}
/*
    trait 只能在对应的crate 里面才能够进行实现，比如trait是在src/lib.rs里面定义的所以，
    对应的impl也只能在lib.rs里面进行实现，不能在main.rs里面实现，
    这样可以防止其他人使用lib的时候恶意破坏lib的代码。
    如下的代码将会报错：
*/
// impl Summary for NewsArticle {
// }


//使用trait限制，只有实现了比较和拷贝的类型才可以call largest函数
fn largest<T: PartialOrd + Copy>(list: &[T]) -> T {
    let mut lag = list[0];
    for &item in list.iter() {
        if lag < item { // PartialOrd trait
            lag = item; // Copy trait
        }
    }
    lag
}
fn largest_test() {
    let number_list = vec![24, 50, 35, 100 ,65];
    let result = largest(&number_list);
    println!("largest is {}", result);
}


struct Point {
    x:i32,
    y:i32
}

use std::fmt;
impl fmt::Display for Point{
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        write!(f, "({}, {})", self.x, self.y)
    }
}


pub trait AddStr {
    fn add_str(&self) -> String;
}

// 为所有支持dispaly trait类型添加一个对于AddStr的支持
// AddStr 这个示例trait的目的是为了加一个前导的字符串
impl <T> AddStr for T
where T: fmt::Display
{
    fn add_str(&self) -> String{
        String::from("head str") + &self.to_string()
    }
}

fn displaytest(){
    let var = Point{
        x:12,
        y:12,
    };

    let str = var.to_string();
    let str1 = var.add_str();
    println!("{} -> {}", str, str1);
}
